unit TBMain;

interface

uses
  WinTypes, WinProcs, Messages,
  Classes, Graphics, Forms, Controls, Menus,
  Dialogs, StdCtrls, Buttons, ExtCtrls,
  SysUtils,
  DCossAPI, TBasicIn, Oxsplits;

type
  TTBasicExample = class(TForm)
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    OpenItem: TMenuItem;
    SaveItem: TMenuItem;
    ExitItem: TMenuItem;
    N1: TMenuItem;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    StatusBar: TPanel;
    SpeedPanel: TPanel;
    OpenBtn: TSpeedButton;
    SaveBtn: TSpeedButton;
    ExitBtn: TSpeedButton;
    BI: TBasicInterpreter;
    Panel1: TPanel;
    Panel2: TPanel;
    biRun: TSpeedButton;
    biStop: TSpeedButton;
    Splitter: ToxSplit;
    Script: TMemo;
    Trace: TMemo;
    Panel4: TPanel;
    TracePanel: TPanel;
    biTrace: TSpeedButton;
    biNew: TSpeedButton;


    procedure ShowHint(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure OpenItemClick(Sender: TObject);
    procedure SaveItemClick(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure biRunClick(Sender: TObject);
    procedure biStopClick(Sender: TObject);
    procedure BINewLine(LineNo: Integer; var Cancel: Boolean);
    procedure FormResize(Sender: TObject);
    procedure BIBasTrace(var Cancel: Boolean;
      var BasTraceState: TBasTraceState);
    procedure BIEndOfText(LineNo: Integer);
    procedure BINextParse(var Cancel: Boolean);
    procedure BISyntaxError(var Continue: Boolean; msg: String);
    procedure biTraceClick(Sender: TObject);
    procedure BIBeforeExec(var Cancel: Boolean);
    procedure FormActivate(Sender: TObject);
    procedure biNewClick(Sender: TObject);
  private
    { Private declarations }
    TraceOn : Boolean;
    procedure SetTrace( switch:boolean );

  public
    { Public declarations }
  end;

var
  TBasicExample: TTBasicExample;

implementation

{uses About;}

Var
  tkTrace : Integer;
  tkOn : Integer;
  tkOff : Integer;

{$R *.DFM}

procedure TTBasicExample.ShowHint(Sender: TObject);
begin
  StatusBar.Caption := Application.Hint;
end;

procedure TTBasicExample.ExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TTBasicExample.OpenItemClick(Sender: TObject);
begin
  if OpenDialog.Execute then begin
    Script.Lines.LoadFromFile(OpenDialog.Filename);
    Caption := 'File: ' + OpenDialog.Filename;
    SaveDialog.Filename := OpenDialog.Filename;
  end;
end;

procedure TTBasicExample.SaveItemClick(Sender: TObject);
begin
  if SaveDialog.Execute then begin
    Script.Lines.SaveToFile(SaveDialog.Filename);
    Caption := 'File: ' + SaveDialog.Filename;
  end;
end;

procedure TTBasicExample.FormPaint(Sender: TObject);
const TabWidth : Integer = 12;
begin
  { set tab stops to ~4 characters }
  Script.Perform(EM_SETTABSTOPS, Word(1), LongInt(@TabWidth));
end;

procedure TTBasicExample.biRunClick(Sender: TObject);
begin
  Trace.Lines.Clear;
  with BI do begin
    { assign the editor's lines to the parser's text property }
    Text.Assign(Script.Lines);
    if Text.Count = 0 then begin
      ShowMessage('Nothing to Execute.');
      Exit;
    end;
    { flip the state of biRun and biStop }
    biRun.Enabled := False;
    biStop.Enabled := True;
    { execute the program }
    Execute := True;
    { flip the execute buttons back }
    biRun.Enabled := True;
    biStop.Enabled := False;
  end;
end;

procedure TTBasicExample.biStopClick(Sender: TObject);
begin
  { tell the parser to stop }
  BI.Execute := False;
  { reset the execution buttons }
  biRun.Enabled := True;
  biStop.Enabled := False;

  StatusBar.Caption := 'Cancelled.';
end;

procedure TTBasicExample.BINewLine(LineNo: Integer; var Cancel: Boolean);
begin
  StatusBar.Caption := 'Run: Line '+IntToStr(LineNo)+
    ' of '+IntToStr(BI.Text.Count);
  Application.ProcessMessages;
end;

procedure TTBasicExample.FormResize(Sender: TObject);
begin
  { try to keep the splitter visible }
  try
    SetTrace(TraceOn);
    {
    if not TraceOn then Splitter.Percent := 99
    else if Splitter.BarPosition > Splitter.Height then
      Splitter.Percent := 75;}
  except
  end;
end;

procedure TTBasicExample.BIBasTrace(var Cancel: Boolean; var BasTraceState: TBasTraceState);
  { return a string of count spaces }
  function spaces(count:integer):String;
  var ii : Integer;
  begin
    Result := '';
    for ii := 1 to count do
      Result := concat(Result,' ');
  end;
  { return a string with tabs packed to a space }
  function pack(txt:String):String;
  var ii : Integer;
  begin
    if txt[0]=#0 then Exit;
    for ii := 1 to byte(txt[0]) do begin
      if txt[ii]=#9 then txt[ii]:=' ';
    end;
    Result:=txt;
  end;
begin
  with BasTraceState, Trace.Lines do begin
    try
      Add(' Action:'+Action+' at '+IntToStr(CurrentLine)+','+IntToStr(CurrentIndex));
      Add(' Text  :'+Pack(LineText));
      Add('========'+spaces(CurrentIndex-1)+'^');
    except
    end;
  end;
end;

procedure TTBasicExample.BIEndOfText(LineNo: Integer);
begin
  StatusBar.Caption :=
    'End of Execution at Line ' + IntToStr(LineNo+1)+
    ', Column ' + IntToStr(bi.CurrentIndex);
end;

procedure TTBasicExample.BINextParse(var Cancel: Boolean);
begin

  with BI do begin
    { adding the Trace On, Trace Off commands }
    if Token = tkTrace then begin
      NextToken;
      if TokenInSetOf( Token, [tkOn, tkOff] ) then begin
        if Token = tkOn
          then SetTrace(True)
          else SetTrace(False);
        Exit;
      end
      else begin
        ErrorMessage('Trace command requires "ON" or "OFF"');
        Cancel := True;
        Exit;
      end;
      Exit;
    end
    { handle the End statement }
    else if Token = tkEnd then begin
      bi.Execute := False;
      Cancel := True;
      Exit;
    end;
  end;

  ShowMessage('Syntax Error: "'+bi.Keyword+
    '" is undefined at Line ' + IntToStr(bi.CurrentLine+1) +
    ', Column '+ IntToStr(bi.CurrentIndex) + '.');
  Cancel := True;
end;

procedure TTBasicExample.SetTrace( switch:boolean );
begin
  TracePanel.Visible := switch;
  Trace.Visible := switch;
  TraceOn := switch;
  bi.TraceBasic := TraceOn;
  if TraceOn
    then Splitter.BarPosition := Splitter.Height div 2
    else Splitter.BarPosition := Splitter.Height;
end;

procedure TTBasicExample.BISyntaxError(var Continue: Boolean; msg: String);
var loc : Word;
begin
  ShowMessage(msg);
  loc := bi.LocateTextAt(bi.CurrentLine, bi.CurrentIndex);
  Script.SelStart := loc-Length(bi.Keyword);
  Script.SelLength := Length(bi.Keyword);
  Continue := False;
end;

procedure TTBasicExample.biTraceClick(Sender: TObject);
begin
  SetTrace(not TraceOn);
end;

procedure TTBasicExample.BIBeforeExec(var Cancel: Boolean);
begin
  with BI do begin
    { remember: TBasicInterpreter uses tokens 100-132 }
    tkTrace := KeyList.AddKey('Trace',True); {token 133}
    tkOn := KeyList.AddKey('On',True);    {token 134}
    tkOff := KeyList.AddKey('Off',True);   {token 135}
  end;
  SetTrace(False);
end;

procedure TTBasicExample.FormActivate(Sender: TObject);
begin
  SetTrace(False);
end;

procedure TTBasicExample.biNewClick(Sender: TObject);
begin
  Script.Lines.Clear;
  Caption := 'TBasic';
  SaveDialog.Filename := '';
end;

end.

